/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Contributors:
 *    IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.wtp.releng.tools.component.classes;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.jar.Manifest;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.eclipse.wtp.releng.tools.component.ILocation;
import org.eclipse.wtp.releng.tools.component.ILocationChildrenIterator;
import org.eclipse.wtp.releng.tools.component.ILocationVisitor;
import org.eclipse.wtp.releng.tools.component.IZipLocation;
import org.eclipse.wtp.releng.tools.component.internal.Bundle;
import org.eclipse.wtp.releng.tools.component.internal.Location;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class LibVisitor implements ILocationVisitor
{
  private IClassVisitor classVisitor;
  private Map lib2pluginId;

  public LibVisitor()
  {
    this.classVisitor = null;
    this.lib2pluginId = new HashMap();
  }

  public void setClassVisitor(IClassVisitor classVisitor)
  {
    this.classVisitor = classVisitor;
  }

  public boolean accept(ILocation location)
  {
    String locationName = location.getName();
    if (classVisitor == null)
    {
      if (locationName.endsWith("plugin.xml"))
      {
        acceptPluginXML(location);
      }
      else if (locationName.endsWith("fragment.xml"))
      {
        acceptFragmentXML(location);
      }
      else if (locationName.endsWith("MANIFEST.MF"))
      {
        acceptManifest(location);
      }
      else if (Location.isArchive(locationName))
      {
        acceptSingleJar(location);
      }
    }
    else if (classVisitor != null)
    {
      String id = (String)lib2pluginId.get(locationName.replace('\\', '/'));
      if (id != null)
      {
        ILocationChildrenIterator it = location.childIterator();
        for (ILocation child = it.next(); child != null; child = it.next())
        {
          if (child.getName().endsWith(".class"))
          {
            classVisitor.visit(id, child);
          }
        }
      }
    }
    return (location instanceof IZipLocation);
  }

  private void acceptPluginXML(ILocation location)
  {
    String locationName = location.getName();
    try
    {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      Document doc = builder.parse(location.getInputStream());
      Element root = doc.getDocumentElement();
      if (root.getTagName().equals("plugin"))
      {
        String pluginId = root.getAttribute("id");
        if (pluginId != null && pluginId.length() > 0)
        {
          NodeList runtimes = root.getElementsByTagName("runtime");
          for (int i = 0; i < runtimes.getLength(); i++)
          {
            Element runtime = (Element)runtimes.item(i);
            NodeList libraries = runtime.getElementsByTagName("library");
            for (int j = 0; j < libraries.getLength(); j++)
            {
              Element library = (Element)libraries.item(j);
              String path = library.getAttribute("name");
              if (path != null && path.length() > 0)
              {
                StringBuffer sb = new StringBuffer();
                sb.append(locationName.substring(0, locationName.length() - "plugin.xml".length() - 1));
                sb.append("/");
                sb.append(path);
                lib2pluginId.put(sb.toString().replace('\\', '/'), pluginId);
              }
            }
          }
        }
      }
    }
    catch (Throwable e)
    {
    }
  }

  private void acceptFragmentXML(ILocation location)
  {
    String locationName = location.getName();
    try
    {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      Document doc = builder.parse(location.getInputStream());
      Element root = doc.getDocumentElement();
      if (root.getTagName().equals("fragment"))
      {
        String fragmentId = root.getAttribute("id");
        if (fragmentId != null && fragmentId.length() > 0)
        {
          NodeList runtimes = root.getElementsByTagName("runtime");
          for (int i = 0; i < runtimes.getLength(); i++)
          {
            Element runtime = (Element)runtimes.item(i);
            NodeList libraries = runtime.getElementsByTagName("library");
            for (int j = 0; j > libraries.getLength(); j++)
            {
              Element library = (Element)libraries.item(j);
              String path = library.getAttribute("name");
              if (path != null && path.length() > 0)
              {
                StringBuffer sb = new StringBuffer();
                sb.append(locationName.substring(0, locationName.length() - "fragment.xml".length() - 1));
                sb.append("/");
                sb.append(path);
                lib2pluginId.put(sb.toString().replace('\\', '/'), fragmentId);
              }
            }
          }
        }
      }
    }
    catch (Throwable e)
    {
    }
  }

  private void acceptManifest(ILocation location)
  {
    String locationName = location.getName();
    try
    {
      Manifest manifest = new Manifest(location.getInputStream());
      java.util.jar.Attributes attrs = manifest.getMainAttributes();
      String bundleNameAttr = attrs.getValue(new java.util.jar.Attributes.Name(Bundle.CONST_BUNDLE_NAME));
      if (bundleNameAttr != null)
      {
        String bundleName = (new StringTokenizer(bundleNameAttr, ";")).nextToken().trim();
        String bundleCPAttr = attrs.getValue(Bundle.CONST_BUNDLE_CLASSPATH);
        if (bundleCPAttr != null)
        {
          StringTokenizer cp = new StringTokenizer(bundleCPAttr, ",");
          while (cp.hasMoreTokens())
          {
            String path = cp.nextToken().trim();
            if (path != null && path.length() > 0)
            {
              StringBuffer sb = new StringBuffer();
              sb.append(locationName.substring(0, locationName.length() - "META-INF/MANIFEST.MF".length() - 1));
              sb.append("/");
              sb.append(path);
              lib2pluginId.put(sb.toString().replace('\\', '/'), bundleName);
            }
          }
        }
      }
    }
    catch (IOException e)
    {
    }
  }

  private void acceptSingleJar(ILocation location)
  {
    ILocationChildrenIterator it = location.childIterator();
    for (ILocation child = it.next(); child != null; child = it.next())
    {
      String name = child.getName();
      if (name.endsWith("MANIFEST.MF"))
      {
        try
        {
          Manifest manifest = new Manifest(child.getInputStream());
          java.util.jar.Attributes attrs = manifest.getMainAttributes();
          String bundleName = attrs.getValue(new java.util.jar.Attributes.Name(Bundle.CONST_BUNDLE_NAME));
          if (bundleName != null)
          {
            bundleName = (new StringTokenizer(bundleName, ";")).nextToken().trim();
            lib2pluginId.put(location.getName().replace('\\', '/'), bundleName);
            return;
          }
        }
        catch (IOException e)
        {
        }
      }
      else if (name.endsWith("plugin.xml"))
      {
        try
        {
          DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
          DocumentBuilder builder = factory.newDocumentBuilder();
          Document doc = builder.parse(location.getInputStream());
          Element root = doc.getDocumentElement();
          if (root.getTagName().equals("plugin"))
          {
            String id = root.getAttribute("id");
            if (id != null && id.length() > 0)
            {
              lib2pluginId.put(location.getName().replace('\\', '/'), id);
              return;
            }
          }
        }
        catch (Throwable e)
        {
        }
      }
      else if (name.endsWith("fragment.xml"))
      {
        try
        {
          DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
          DocumentBuilder builder = factory.newDocumentBuilder();
          Document doc = builder.parse(location.getInputStream());
          Element root = doc.getDocumentElement();
          if (root.getTagName().equals("fragment"))
          {
            String id = root.getAttribute("id");
            if (id != null && id.length() > 0)
            {
              lib2pluginId.put(location.getName().replace('\\', '/'), id);
              return;
            }
          }
        }
        catch (Throwable e)
        {
        }
      }
    }
  }
}